@@ -25,8 +25,9 @@
 #include <fcitx-utils/key.h>
 #include <fcitx-utils/keysym.h>
 #include <fcitx-utils/misc.h>
-#include <fcitx-utils/standardpath.h>
+#include <fcitx-utils/standardpaths.h>
 #include <fcitx-utils/trackableobject.h>
+#include <fcitx-utils/unixfd.h>
 #include <fcitx/action.h>
 #include <fcitx/addoninstance.h>
 #include <fcitx/addonmanager.h>
@@ -36,6 +37,7 @@
 #include <fcitx/inputmethodengine.h>
 #include <fcitx/instance.h>
 #include <fcitx/text.h>
+#include <filesystem>
 #include <future>
 #include <libime/pinyin/pinyincontext.h>
 #include <libime/pinyin/pinyinime.h>
@@ -96,6 +98,17 @@ enum class PreeditMode { No, ComposingPinyin, CommitPreview };
 FCITX_CONFIG_ENUM_NAME_WITH_I18N(PreeditMode, N_("Do not show"),
                                  N_("Composing pinyin"), N_("Commit preview"))
 
+enum class BackspaceBehaviorOnPrediction {
+    OnlyClearCandidates,
+    ClearCandidatesAndBackspace,
+    BackspaceWhenNotUsingOnScreenKeyboard
+};
+
+FCITX_CONFIG_ENUM_NAME_WITH_I18N(
+    BackspaceBehaviorOnPrediction, N_("Only Clear candidates"),
+    N_("Clear candidates & backspace"),
+    N_("Backspace when not using on-screen keyboard"))
+
 enum class CorrectionLayout {
     None,
     Qwerty,
@@ -126,6 +139,7 @@ FCITX_CONFIGURATION(
     Option<bool> c{this, "C_CH", _("c <-> ch"), false};
     Option<bool> f{this, "F_H", _("f <-> h"), false};
     Option<bool> l{this, "L_N", _("l <-> n"), false};
+    Option<bool> r{this, "L_R", _("l <-> r"), false};
     Option<bool> s{this, "S_SH", _("s <-> sh"), false};
     Option<bool> z{this, "Z_ZH", _("z <-> zh"), false};
     OptionWithAnnotation<CorrectionLayout, CorrectionLayoutI18NAnnotation>
@@ -142,22 +156,31 @@ FCITX_CONFIGURATION(
                          ShuangpinProfileEnum::Ziranma};
     OptionWithAnnotation<bool, OptionalHideInDescription> showShuangpinMode{
         this, "ShowShuangpinMode", _("Show current shuangpin mode"), true};
-    Option<int, IntConstrain> pageSize{this, "PageSize", _("Page size"), 7,
-                                       IntConstrain(3, 10)};
-    Option<bool> spellEnabled{this, "SpellEnabled", _("Enable Spell"), true};
-    Option<bool> symbolsEnabled{this, "SymbolsEnabled", _("Enable Symbols"),
-                                true};
-    Option<bool> chaiziEnabled{this, "ChaiziEnabled", _("Enable Chaizi"), true};
-    Option<bool> extBEnabled{this, "ExtBEnabled",
-                             _("Enable Characters in Unicode CJK Extension B"),
-                             !isAndroid()};
-    OptionalHiddenSubConfigOption cloudpinyin{
-        this, "CloudPinyin", _("Cloud Pinyin"),
-        "fcitx://config/addon/cloudpinyin"};
+    Option<int, IntConstrain> pageSize{
+        this, "PageSize", _("Candidates Per Page"), 7, IntConstrain(3, 10)};
+    Option<bool> spellEnabled{this, "SpellEnabled",
+                              _("Show English Candidates"), true};
+    Option<bool> symbolsEnabled{this, "SymbolsEnabled",
+                                _("Show symbol candidates"), true};
+    Option<bool> chaiziEnabled{this, "ChaiziEnabled",
+                               _("Show Chaizi candidates"), true};
+    Option<bool> extBEnabled{
+        this, "ExtBEnabled",
+        _("Enable more Characters after Unicode CJK Extension B"),
+        !isAndroid()};
+    Option<bool> strokeCandidateEnabled{
+        this, "StrokeCandidateEnabled",
+        _("Show stroke candidates when typing with h(一), s(丨), p(丿), n(㇏), "
+          "z(𠃍)"),
+        true};
     OptionWithAnnotation<bool, OptionalHideInDescription> cloudPinyinEnabled{
         this, "CloudPinyinEnabled", _("Enable Cloud Pinyin"), false};
+    OptionalHiddenSubConfigOption cloudpinyin{
+        this, "CloudPinyin", _("Configure Cloud Pinyin"),
+        "fcitx://config/addon/cloudpinyin"};
     Option<int, IntConstrain, DefaultMarshaller<int>, OptionalHideInDescription>
-        cloudPinyinIndex{this, "CloudPinyinIndex", _("Cloud Pinyin Index"), 2,
+        cloudPinyinIndex{this, "CloudPinyinIndex",
+                         _("Cloud Pinyin Candidate Order"), 2,
                          IntConstrain(1, 10)};
     OptionWithAnnotation<bool, OptionalHideInDescription> cloudPinyinAnimation{
         this, "CloudPinyinAnimation",
@@ -177,8 +200,24 @@ FCITX_CONFIGURATION(
         this, "PinyinInPreedit", _("Show complete pinyin in preedit"), false};
     Option<bool> predictionEnabled{this, "Prediction", _("Enable Prediction"),
                                    isAndroid()};
-    Option<int, IntConstrain> predictionSize{
-        this, "PredictionSize", _("Prediction Size"), 49, IntConstrain(3, 100)};
+    OptionWithAnnotation<bool, ToolTipAnnotation> keepCurrentContext{
+        this,
+        "KeepCurrentContext",
+        _("Keep the current typed text for next input prediction"),
+        true,
+        {},
+        {},
+        {_("Keep the current typed text for future prediction.")}};
+    Option<int, IntConstrain> predictionSize{this, "PredictionSize",
+                                             _("Number of Predictions"), 49,
+                                             IntConstrain(3, 100)};
+    OptionWithAnnotation<BackspaceBehaviorOnPrediction,
+                         BackspaceBehaviorOnPredictionI18NAnnotation>
+        backspaceBehaviorOnPrediction{
+            this, "BackspaceBehaviorOnPrediction",
+            _("Backspace behavior on prediction"),
+            BackspaceBehaviorOnPrediction::
+                BackspaceWhenNotUsingOnScreenKeyboard};
     OptionWithAnnotation<SwitchInputMethodBehavior,
                          SwitchInputMethodBehaviorI18NAnnotation>
         switchInputMethodBehavior{this, "SwitchInputMethodBehavior",
@@ -232,14 +271,14 @@ FCITX_CONFIGURATION(
     KeyListOption secondCandidate{
         this,
         "SecondCandidate",
-        _("Select 2nd Candidate"),
+        _("Select Second Candidate"),
         {},
         KeyListConstrain({KeyConstrainFlag::AllowModifierLess,
                           KeyConstrainFlag::AllowModifierOnly})};
     KeyListOption thirdCandidate{
         this,
         "ThirdCandidate",
-        _("Select 3rd Candidate"),
+        _("Select Third Candidate"),
         {},
         KeyListConstrain({KeyConstrainFlag::AllowModifierLess,
                           KeyConstrainFlag::AllowModifierOnly})};
@@ -263,6 +302,14 @@ FCITX_CONFIGURATION(
     Option<int, IntConstrain> nbest{this, "Number of sentence",
                                     _("Number of Sentences"), 2,
                                     IntConstrain(1, 3)};
+    Option<int, IntConstrain, DefaultMarshaller<int>, ToolTipAnnotation>
+        wordCandidateLimit{this,
+                           "WordCandidateLimit",
+                           _("Number of Phrase Candidates"),
+                           15,
+                           IntConstrain(0),
+                           {},
+                           {_("Set to 0 will show all candidates.")}};
     Option<int, IntConstrain> longWordLimit{
         this, "LongWordLengthLimit",
         _("Prompt long word length when input length over (0 for disable)"), 4,
@@ -275,7 +322,8 @@ FCITX_CONFIGURATION(
         this, "Punctuation", _("Punctuation"),
         "fcitx://config/addon/punctuation/punctuationmap/zh_CN"};
     SubConfigOption chttrans{
-        this, "Chttrans", _("Simplified and Traditional Chinese Translation"),
+        this, "Chttrans",
+        _("Configure Simplified/Traditional Chinese Conversion"),
         "fcitx://config/addon/chttrans"};
     Option<Key, KeyConstrain> quickphraseKey{
         this,
@@ -298,7 +346,7 @@ FCITX_CONFIGURATION(
             {},
             {_("Enter quickphrase mode when current input matches any regular "
                "expression from the list.")}};
-    Option<FuzzyConfig> fuzzyConfig{this, "Fuzzy", _("Fuzzy Pinyin Settings")};
+    Option<FuzzyConfig> fuzzyConfig{this, "Fuzzy", _("Fuzzy Pinyin")};
     HiddenOption<bool> firstRun{this, "FirstRun", "FirstRun", true};)
 
 struct EventSourceTime;
@@ -430,7 +478,7 @@ private:
     void loadBuiltInDict();
     void loadExtraDict();
     void loadCustomPhrase();
-    void loadSymbols(const StandardPathFile &file);
+    void loadSymbols(const UnixFD &file);
     void loadDict(const std::string &fullPath,
                   std::list<std::unique_ptr<TaskToken>> &taskTokens);
     void saveCustomPhrase();
