@@ -8,10 +8,24 @@
 #define _CHTTRANS_CHTTRANS_NATIVE_H_
 
 #include "chttrans.h"
+#include <functional>
+#include <string>
+#include <string_view>
 #include <unordered_map>
 
 class NativeBackend : public ChttransBackend {
+    struct string_hash {
+        using is_transparent = void;
+
+        size_t operator()(const std::string_view str) const {
+            constexpr std::hash<std::string_view> hasher{};
+            return hasher(str);
+        }
+    };
+
 public:
+    using MapType = std::unordered_map<std::string, std::string, string_hash,
+                                       std::equal_to<>>;
     std::string convertSimpToTrad(const std::string &) override;
     std::string convertTradToSimp(const std::string &) override;
 
@@ -19,8 +33,8 @@ protected:
     bool loadOnce(const ChttransConfig &) override;
 
 private:
-    std::unordered_map<uint32_t, std::string> s2tMap_;
-    std::unordered_map<uint32_t, std::string> t2sMap_;
+    MapType s2tMap_;
+    MapType t2sMap_;
 };
 
 #endif // _CHTTRANS_CHTTRANS_NATIVE_H_
